"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const compiler_1 = require("../query/compiler");
class PostgresEngine {
    constructor(pool) {
        this.pool = pool;
    }
    query(sql, params) {
        return this.pool.query(this.transformQuery(sql, params)).then(() => { });
    }
    get(sql, params) {
        return this.pool.query(this.transformQuery(sql, params)).then(x => x.rows);
    }
    insertAndGetId(table, sql, params) {
        return this.get(sql + " RETURNING id", params).then(x => x[0]["id"]);
    }
    getGrammarCompiler() {
        return new class extends compiler_1.default {
            escapeColumn(column) {
                if (column === "*")
                    return column;
                // We need to escape both parts of the as independently.
                if (column.toLowerCase().indexOf(" as ") !== -1) {
                    const [original, , alias] = column.split(" ");
                    return this.escapeColumn(original) + " AS " + this.escapeColumn(alias);
                }
                return column.split(".").map(x => '"' + x + '"').join(".");
            }
            escapeTable(table) {
                return table.split(".").map(x => '"' + x + '"').join(".");
            }
        };
    }
    /**
     * Converts ? placeholders into numbered $# placeholders that postgres understands.
     */
    transformQuery(sql, params) {
        let i = 1;
        return {
            text: sql.replace(/\?/g, () => "$" + (i++)),
            values: params
        };
    }
}
exports.default = PostgresEngine;
