import { Database as sqliteDatabase } from "sqlite3";
import { DatabaseEngine } from "./database-engine";
import * as pg from "pg";
import * as mysql from "mysql2/promise";
import BaseModel, { Wrapped } from "./base-model";
export declare class BasieStatic {
    private engine;
    /**
     * Returns the engine currently in use, or throws if there is no engine configured.
     */
    getEngine(): DatabaseEngine;
    /**
     * Changes the current database engine to the specified engine.
     */
    use(engine: DatabaseEngine): void;
    /**
     * Configures basie to be used with the specified sqlite database.
     */
    sqlite(db: sqliteDatabase): void;
    /**
     * Configures basie to be used with the specified postgres connection pool.
     */
    postgres(db: pg.Pool): void;
    /**
     * Configures basie to be used with the specified mysql connection.
     */
    mysql(connection: mysql.Connection): void;
    /**
     * Wraps the specified model with basie-specific methods that give it static
     * querying methods. This does not create a table within the database, and
     * instead expects the table to already be present. This function is called
     * a bit weirdly (`Basie.wrap<_Model>()(_Model)`) to overcome some typescript
     * type system limits and statically typecheck that you're only using valid
     * members.
     */
    wrap<InstanceType extends BaseModel, Fields extends string = keyof InstanceType>(): <ConstructorFN extends new (...args: any[]) => InstanceType2, InstanceType2 extends Record<Fields, string | number | boolean | Function>>(arg: ConstructorFN, tableName?: string | undefined) => Wrapped<ConstructorFN, InstanceType>;
}
declare const _default: BasieStatic;
export default _default;
export { default as BaseModel, R, A } from "./base-model";
