import QueryBuilder from "./builder";
import { JoinType, Operator, QueryBoolean } from "./types";
/**
 * Represents a QueryBuilder for a nested join clause.
 * We essentially abuse QueryBuilder's WHERE building since it works
 * exactly the same as the ON clause of a join.
 */
export default class JoinClause extends QueryBuilder<any> {
    type: JoinType;
    joiningOn: string;
    constructor(type: JoinType, joiningOn: string);
    /**
     * Creates a new nested ON clause. The specified callback receives a nested
     * query builder that can be used to build the nested queries.
     */
    on(method: (builder: this) => any): this;
    /**
     * Adds a new ON clause for the specified column, operator and value. Optionally
     * you can provide a QueryBoolean of "OR" instead of "AND", but it is recommended that
     * you use `orOn` instead.
     */
    on(first: string, operator: Operator, second: string, boolean?: QueryBoolean): this;
    /**
     * Creates a new nested OR ON clause. The specified callback receives a nested
     * query builder that can be used to build the nested queries.
     */
    orOn(method: () => any): this;
    /**
     * Adds a new OR ON clause for the specified column, operator and value.
     */
    orOn(first: string, operator: Operator, second: string): this;
    protected createNew(): this;
}
