"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const builder_1 = require("./builder");
/**
 * Represents a QueryBuilder for a nested join clause.
 * We essentially abuse QueryBuilder's WHERE building since it works
 * exactly the same as the ON clause of a join.
 */
class JoinClause extends builder_1.default {
    constructor(type, joiningOn) {
        super();
        this.table = joiningOn;
        this.type = type;
        this.joiningOn = joiningOn;
    }
    on(firstOrMethod, operator, second, boolean = "AND") {
        if (typeof firstOrMethod === "function") {
            return this.whereNested(firstOrMethod, boolean);
        }
        return this.whereColumn(firstOrMethod, operator, second, boolean);
    }
    orOn(firstOrMethod, operator, second) {
        return this.on(firstOrMethod, operator, second, "OR");
    }
    createNew() {
        return new JoinClause(this.type, this.joiningOn);
    }
}
exports.default = JoinClause;
