"use strict";
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __metadata = (this && this.__metadata) || function (k, v) {
    if (typeof Reflect === "object" && typeof Reflect.metadata === "function") return Reflect.metadata(k, v);
};
Object.defineProperty(exports, "__esModule", { value: true });
const mocha_typescript_1 = require("mocha-typescript");
const chai_1 = require("chai");
const builder_1 = require("../query/builder");
const compiler_1 = require("../query/compiler");
function compileSelect(fn, toEq, args) {
    const builder = builder_1.default.table("test");
    fn(builder);
    const built = new compiler_1.default().compileSelect(builder);
    chai_1.expect(built.sql).to.equal(toEq);
    if (args)
        chai_1.expect(built.args).to.deep.equal(args);
}
let SQLGrammarCompilerTests = class SQLGrammarCompilerTests {
    compilesAggregateFunctions() {
        compileSelect(x => x.aggregateFunction = "SUM", "SELECT SUM(*) AS aggregate FROM test");
        compileSelect(x => x.aggregateFunction = "AVG", "SELECT AVG(*) AS aggregate FROM test");
        compileSelect(x => x.aggregateFunction = "MIN", "SELECT MIN(*) AS aggregate FROM test");
        compileSelect(x => x.aggregateFunction = "MAX", "SELECT MAX(*) AS aggregate FROM test");
        compileSelect(x => {
            x.aggregateFunction = "MAX";
            x.select("bar");
            x.distinct();
        }, "SELECT MAX(DISTINCT bar) AS aggregate FROM test");
        compileSelect(x => {
            x.aggregateFunction = "MAX";
            x.distinct();
        }, "SELECT MAX(*) AS aggregate FROM test");
    }
    compilesArbitraryColumns() {
        compileSelect(x => x.select("foo", "bar AS baz"), "SELECT foo, bar AS baz FROM test");
    }
    compilesDistinctColumns() {
        compileSelect(x => x.distinct(), "SELECT DISTINCT * FROM test");
    }
    compilesJoins() {
        compileSelect(x => x.join("foo", "test.foo", "=", "foo.bar"), "SELECT * FROM test INNER JOIN foo ON test.foo = foo.bar");
        compileSelect(x => x.leftJoin("foo", "test.foo", "=", "foo.bar"), "SELECT * FROM test LEFT JOIN foo ON test.foo = foo.bar");
        compileSelect(x => x.rightJoin("foo", "test.foo", "=", "foo.bar"), "SELECT * FROM test RIGHT JOIN foo ON test.foo = foo.bar");
        compileSelect(x => x.join("foo", nested => {
            nested.on("test.a", "=", "foo.b");
            nested.orOn("test.b", "=", "foo.c");
        }), "SELECT * FROM test INNER JOIN foo ON test.a = foo.b OR test.b = foo.c");
        compileSelect(x => x.join("foo", nested => {
            nested.on(evenMoreNested => evenMoreNested.on("a", "=", "b").orOn("b", "=", "c"));
            nested.orOn("test.b", "=", "foo.c");
        }), "SELECT * FROM test INNER JOIN foo ON (a = b OR b = c) OR test.b = foo.c");
    }
    compilesBasicWheres() {
        compileSelect(x => x.where("foo", "10"), "SELECT * FROM test WHERE foo = ?", ["10"]);
        compileSelect(x => x.where("foo", "10").orWhere("bar", 200), "SELECT * FROM test WHERE foo = ? OR bar = ?", ["10", 200]);
    }
    compilesColumnWheres() {
        compileSelect(x => x.whereColumn("foo", "=", "bar"), "SELECT * FROM test WHERE foo = bar");
        compileSelect(x => x.whereColumn("foo", "=", "bar").orWhereColumn("foo", "<", "bar"), "SELECT * FROM test WHERE foo = bar OR foo < bar");
    }
    compilesRawWheres() {
        compileSelect(x => x.whereRaw("a LIKE ?", ["Foo"]), "SELECT * FROM test WHERE a LIKE ?", ["Foo"]);
        compileSelect(x => x.whereRaw("a LIKE ?", ["Foo"]).orWhereRaw("b LIKE ?", [10]), "SELECT * FROM test WHERE a LIKE ? OR b LIKE ?", ["Foo", 10]);
    }
    compilesNullWheres() {
        compileSelect(x => x.whereNull("a"), "SELECT * FROM test WHERE a IS NULL");
        compileSelect(x => x.whereNotNull("a"), "SELECT * FROM test WHERE a IS NOT NULL");
        compileSelect(x => x.whereNull("a").orWhereNull("b"), "SELECT * FROM test WHERE a IS NULL OR b IS NULL");
        compileSelect(x => x.whereNull("a").orWhereNotNull("b"), "SELECT * FROM test WHERE a IS NULL OR b IS NOT NULL");
    }
    compilesNestedWheres() {
        compileSelect(x => x.where("a", "A").orWhere(nested => {
            nested.where("a", "B");
            nested.where("b", 10);
        }), "SELECT * FROM test WHERE a = ? OR (a = ? AND b = ?)", ["A", "B", 10]);
        compileSelect(x => x.where("a", "A").orWhere(nested => {
            nested.where("a", "B");
            nested.orWhereNested(n => n.where("b", 10));
        }), "SELECT * FROM test WHERE a = ? OR (a = ? OR (b = ?))", ["A", "B", 10]);
    }
    compilesGroups() {
        compileSelect(x => x.groupBy("a"), "SELECT * FROM test GROUP BY a");
        compileSelect(x => x.groupBy("a", "b"), "SELECT * FROM test GROUP BY a, b");
    }
    compilesOrders() {
        compileSelect(x => x.orderBy("a"), "SELECT * FROM test ORDER BY a ASC");
        compileSelect(x => x.orderByAsc("a"), "SELECT * FROM test ORDER BY a ASC");
        compileSelect(x => x.orderByDesc("a"), "SELECT * FROM test ORDER BY a DESC");
        compileSelect(x => x.orderBy("a").orderByDesc("b"), "SELECT * FROM test ORDER BY a ASC, b DESC");
    }
    compilesLimits() {
        compileSelect(x => x.limit(10), "SELECT * FROM test LIMIT ?", [10]);
    }
    compilesInserts() {
        const builder = builder_1.default.table("test");
        let result = new compiler_1.default().compileInsert(builder, [{ a: "Foo", b: 10 }]);
        chai_1.expect(result.sql).to.equal("INSERT INTO test (a,b) VALUES (?,?)");
        chai_1.expect(result.args).to.deep.equal(["Foo", 10]);
        result = new compiler_1.default().compileInsert(builder, [{ a: "Foo", b: 10 }, { a: "Bar", b: 12 }]);
        chai_1.expect(result.sql).to.equal("INSERT INTO test (a,b) VALUES (?,?), (?,?)");
        chai_1.expect(result.args).to.deep.equal(["Foo", 10, "Bar", 12]);
    }
    compilesUpdates() {
        const builder = builder_1.default.table("test");
        builder.where("b", 10);
        let result = new compiler_1.default().compileUpdate(builder, { a: "Foo" });
        chai_1.expect(result.sql).to.equal("UPDATE test SET a = ? WHERE b = ?");
        chai_1.expect(result.args).to.deep.equal(["Foo", 10]);
        result = new compiler_1.default().compileUpdate(builder, { a: "Foo", b: 12 });
        chai_1.expect(result.sql).to.equal("UPDATE test SET a = ?, b = ? WHERE b = ?");
        chai_1.expect(result.args).to.deep.equal(["Foo", 12, 10]);
        builder.join("foo", "foo.a", "=", "test.b");
        result = new compiler_1.default().compileUpdate(builder, { a: "Foo", b: 12 });
        chai_1.expect(result.sql).to.equal("UPDATE test INNER JOIN foo ON foo.a = test.b SET a = ?, b = ? WHERE b = ?");
        chai_1.expect(result.args).to.deep.equal(["Foo", 12, 10]);
    }
    compilesDeletes() {
        const builder = builder_1.default.table("test");
        let result = new compiler_1.default().compileDelete(builder);
        chai_1.expect(result.sql).to.equal("DELETE FROM test");
        result = new compiler_1.default().compileDelete(builder.where("b", 10));
        chai_1.expect(result.sql).to.equal("DELETE FROM test WHERE b = ?");
        chai_1.expect(result.args).to.deep.equal([10]);
    }
};
__decorate([
    mocha_typescript_1.test,
    __metadata("design:type", Function),
    __metadata("design:paramtypes", []),
    __metadata("design:returntype", void 0)
], SQLGrammarCompilerTests.prototype, "compilesAggregateFunctions", null);
__decorate([
    mocha_typescript_1.test,
    __metadata("design:type", Function),
    __metadata("design:paramtypes", []),
    __metadata("design:returntype", void 0)
], SQLGrammarCompilerTests.prototype, "compilesArbitraryColumns", null);
__decorate([
    mocha_typescript_1.test,
    __metadata("design:type", Function),
    __metadata("design:paramtypes", []),
    __metadata("design:returntype", void 0)
], SQLGrammarCompilerTests.prototype, "compilesDistinctColumns", null);
__decorate([
    mocha_typescript_1.test,
    __metadata("design:type", Function),
    __metadata("design:paramtypes", []),
    __metadata("design:returntype", void 0)
], SQLGrammarCompilerTests.prototype, "compilesJoins", null);
__decorate([
    mocha_typescript_1.test,
    __metadata("design:type", Function),
    __metadata("design:paramtypes", []),
    __metadata("design:returntype", void 0)
], SQLGrammarCompilerTests.prototype, "compilesBasicWheres", null);
__decorate([
    mocha_typescript_1.test,
    __metadata("design:type", Function),
    __metadata("design:paramtypes", []),
    __metadata("design:returntype", void 0)
], SQLGrammarCompilerTests.prototype, "compilesColumnWheres", null);
__decorate([
    mocha_typescript_1.test,
    __metadata("design:type", Function),
    __metadata("design:paramtypes", []),
    __metadata("design:returntype", void 0)
], SQLGrammarCompilerTests.prototype, "compilesRawWheres", null);
__decorate([
    mocha_typescript_1.test,
    __metadata("design:type", Function),
    __metadata("design:paramtypes", []),
    __metadata("design:returntype", void 0)
], SQLGrammarCompilerTests.prototype, "compilesNullWheres", null);
__decorate([
    mocha_typescript_1.test,
    __metadata("design:type", Function),
    __metadata("design:paramtypes", []),
    __metadata("design:returntype", void 0)
], SQLGrammarCompilerTests.prototype, "compilesNestedWheres", null);
__decorate([
    mocha_typescript_1.test,
    __metadata("design:type", Function),
    __metadata("design:paramtypes", []),
    __metadata("design:returntype", void 0)
], SQLGrammarCompilerTests.prototype, "compilesGroups", null);
__decorate([
    mocha_typescript_1.test,
    __metadata("design:type", Function),
    __metadata("design:paramtypes", []),
    __metadata("design:returntype", void 0)
], SQLGrammarCompilerTests.prototype, "compilesOrders", null);
__decorate([
    mocha_typescript_1.test,
    __metadata("design:type", Function),
    __metadata("design:paramtypes", []),
    __metadata("design:returntype", void 0)
], SQLGrammarCompilerTests.prototype, "compilesLimits", null);
__decorate([
    mocha_typescript_1.test,
    __metadata("design:type", Function),
    __metadata("design:paramtypes", []),
    __metadata("design:returntype", void 0)
], SQLGrammarCompilerTests.prototype, "compilesInserts", null);
__decorate([
    mocha_typescript_1.test,
    __metadata("design:type", Function),
    __metadata("design:paramtypes", []),
    __metadata("design:returntype", void 0)
], SQLGrammarCompilerTests.prototype, "compilesUpdates", null);
__decorate([
    mocha_typescript_1.test,
    __metadata("design:type", Function),
    __metadata("design:paramtypes", []),
    __metadata("design:returntype", void 0)
], SQLGrammarCompilerTests.prototype, "compilesDeletes", null);
SQLGrammarCompilerTests = __decorate([
    mocha_typescript_1.suite
], SQLGrammarCompilerTests);
