"use strict";
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __metadata = (this && this.__metadata) || function (k, v) {
    if (typeof Reflect === "object" && typeof Reflect.metadata === "function") return Reflect.metadata(k, v);
};
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const mocha_typescript_1 = require("mocha-typescript");
const chai_1 = require("chai");
const mysql = require("mysql2/promise");
const _1 = require("../");
const builder_1 = require("../query/builder");
function builder() {
    return builder_1.default.table("users");
}
// Skip tests if no postgres connection is defined.
const test = process.env.MYSQL_HOST ? mocha_typescript_1.test : mocha_typescript_1.test.skip;
let MySQLTests = class MySQLTests {
    before() {
        return __awaiter(this, void 0, void 0, function* () {
            if (!process.env.MYSQL_HOST)
                return;
            const connection = yield mysql.createConnection({
                user: process.env.MYSQL_USER,
                password: process.env.MYSQL_PASSWORD,
                database: process.env.MYSQL_DATABASE,
                host: process.env.MYSQL_HOST,
                port: +process.env.MYSQL_PORT,
                decimalNumbers: true
            });
            yield connection.query("DROP TABLE IF EXISTS users");
            yield connection.query("CREATE TABLE users (id int NOT NULL AUTO_INCREMENT PRIMARY KEY, name TEXT, age INTEGER)");
            _1.default.mysql(connection);
        });
    }
    after() {
        return __awaiter(this, void 0, void 0, function* () {
            // Ensure that database is clean unset after our tests.
            _1.default.use(null);
        });
    }
    inserts() {
        return __awaiter(this, void 0, void 0, function* () {
            yield builder().insert({ name: "Thijs", age: 17 });
            chai_1.expect(yield builder().exists()).to.equal(true);
            chai_1.expect(yield builder().all()).to.have.length(1);
        });
    }
    updates() {
        return __awaiter(this, void 0, void 0, function* () {
            yield builder().insert({ name: "Thijs", age: 17 });
            yield builder().where("id", 1).update({ age: 18 });
            chai_1.expect(yield builder().where("id", 1).value("age")).to.equal(18);
        });
    }
    deletes() {
        return __awaiter(this, void 0, void 0, function* () {
            yield builder().insert({ name: "Thijs", age: 17 });
            yield builder().where("name", "Thijs").delete();
            chai_1.expect(yield builder().count()).to.equal(0);
        });
    }
    aggregates() {
        return __awaiter(this, void 0, void 0, function* () {
            yield builder().insert({ name: "Thijs", age: 17 }, { name: "Silke", age: 15 }, { name: "Marcel", age: 52 }, { name: "Christien", age: 50 });
            chai_1.expect(yield builder().count()).to.equal(4);
            chai_1.expect(yield builder().min("age")).to.equal(15);
            chai_1.expect(yield builder().max("age")).to.equal(52);
            chai_1.expect(yield builder().avg("age")).to.equal(33.5);
            chai_1.expect(yield builder().sum("age")).to.equal(134);
        });
    }
    fetching() {
        return __awaiter(this, void 0, void 0, function* () {
            yield builder().insert({ name: "Thijs", age: 17 }, { name: "Silke", age: 15 }, { name: "Marcel", age: 52 }, { name: "Christien", age: 50 });
            chai_1.expect(yield builder().pluck("name")).to.deep.equal(["Thijs", "Silke", "Marcel", "Christien"]);
            chai_1.expect(yield builder().first()).property("name").to.equal("Thijs");
            chai_1.expect(yield builder().where("name", "LIKE", "%i%").count()).to.equal(3);
        });
    }
    escapesAsProperly() {
        return __awaiter(this, void 0, void 0, function* () {
            yield builder().insert({ name: "Thijs", age: 17 });
            chai_1.expect(yield builder().select("age AS myAge").value("myAge")).to.equal(17);
        });
    }
    insertingWithId() {
        return __awaiter(this, void 0, void 0, function* () {
            chai_1.expect(yield _1.default.getEngine().insertAndGetId("users", "INSERT INTO users (name, age) VALUES (?, ?)", ["Thijs", 17])).to.equal(1);
            chai_1.expect(yield _1.default.getEngine().insertAndGetId("users", "INSERT INTO users (name, age) VALUES (?, ?)", ["Silke", 15])).to.equal(2);
        });
    }
};
__decorate([
    test,
    __metadata("design:type", Function),
    __metadata("design:paramtypes", []),
    __metadata("design:returntype", Promise)
], MySQLTests.prototype, "inserts", null);
__decorate([
    test,
    __metadata("design:type", Function),
    __metadata("design:paramtypes", []),
    __metadata("design:returntype", Promise)
], MySQLTests.prototype, "updates", null);
__decorate([
    test,
    __metadata("design:type", Function),
    __metadata("design:paramtypes", []),
    __metadata("design:returntype", Promise)
], MySQLTests.prototype, "deletes", null);
__decorate([
    test,
    __metadata("design:type", Function),
    __metadata("design:paramtypes", []),
    __metadata("design:returntype", Promise)
], MySQLTests.prototype, "aggregates", null);
__decorate([
    test,
    __metadata("design:type", Function),
    __metadata("design:paramtypes", []),
    __metadata("design:returntype", Promise)
], MySQLTests.prototype, "fetching", null);
__decorate([
    test,
    __metadata("design:type", Function),
    __metadata("design:paramtypes", []),
    __metadata("design:returntype", Promise)
], MySQLTests.prototype, "escapesAsProperly", null);
__decorate([
    test,
    __metadata("design:type", Function),
    __metadata("design:paramtypes", []),
    __metadata("design:returntype", Promise)
], MySQLTests.prototype, "insertingWithId", null);
MySQLTests = __decorate([
    mocha_typescript_1.suite
], MySQLTests);
