"use strict";
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __metadata = (this && this.__metadata) || function (k, v) {
    if (typeof Reflect === "object" && typeof Reflect.metadata === "function") return Reflect.metadata(k, v);
};
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const mocha_typescript_1 = require("mocha-typescript");
const chai_1 = require("chai");
const sqlite3_1 = require("sqlite3");
const _1 = require("../");
const base_model_1 = require("../base-model");
const builder_1 = require("../query/builder");
class _User extends base_model_1.default {
    constructor() {
        super(...arguments);
        this.phone = this.hasOne(Phone);
        this.phones = this.hasMany(Phone);
        this.allPhones = this.hasAndBelongsToMany(Phone);
    }
}
const User = _1.default.wrap()(_User);
class _Phone extends base_model_1.default {
    constructor() {
        super(...arguments);
        this.hasUser = this.hasOne(User);
        this.user = this.belongsTo(User);
        this.allUsers = this.hasAndBelongsToMany(User);
    }
}
const Phone = _1.default.wrap()(_Phone);
let RelationTests = class RelationTests {
    before() {
        return __awaiter(this, void 0, void 0, function* () {
            let db;
            yield new Promise(resolve => db = new sqlite3_1.Database(":memory:", () => resolve()));
            _1.default.sqlite(db);
            yield builder_1.default.execute("CREATE TABLE users (id INTEGER NOT NULL PRIMARY KEY AUTOINCREMENT UNIQUE, name TEXT, age INTEGER, phone_id INTEGER)");
            yield builder_1.default.execute("CREATE TABLE phones (id INTEGER NOT NULL PRIMARY KEY AUTOINCREMENT UNIQUE, number TEXT, user_id INTEGER)");
            yield builder_1.default.execute("CREATE TABLE phones_users (user_id INTEGER, phone_id INTEGER)");
        });
    }
    after() {
        return __awaiter(this, void 0, void 0, function* () {
            // Ensure that database is clean unset after our tests.
            _1.default.use(null);
        });
    }
    hasOne() {
        return __awaiter(this, void 0, void 0, function* () {
            yield User.insert({ name: "Thijs", age: 17, phone_id: 1 });
            yield Phone.insert({ number: "123", user_id: 1 });
            const user = (yield User.first());
            const phone = yield user.phone();
            chai_1.expect(phone.number).to.equal("123");
            const back = yield phone.hasUser();
            chai_1.expect(back.name).to.equal("Thijs");
            const phone2 = yield user.phone();
            chai_1.expect(phone).to.equal(phone2); // lazy loading
        });
    }
    belongsTo() {
        return __awaiter(this, void 0, void 0, function* () {
            yield User.insert({ name: "Thijs", age: 17, phone_id: 1 });
            yield Phone.insert({ number: "123", user_id: 1 });
            const user = (yield User.first());
            const back = yield user.phone().then(x => x.user());
            chai_1.expect(user.name).to.equal(back.name);
            chai_1.expect(user.id).to.equal(back.id);
        });
    }
    hasMany() {
        return __awaiter(this, void 0, void 0, function* () {
            yield User.insert({ name: "Thijs", age: 17, phone_id: 1 });
            yield Phone.insert({ number: "123", user_id: 1 });
            yield Phone.insert({ number: "456", user_id: 1 });
            const user = (yield User.first());
            const phones = yield user.phones();
            chai_1.expect(phones).to.have.length(2);
            chai_1.expect(phones.map(x => x.number)).to.eql(["123", "456"]);
        });
    }
    hasAndBelongsToMany() {
        return __awaiter(this, void 0, void 0, function* () {
            yield User.insert({ name: "Thijs", age: 17, phone_id: 1 });
            yield User.insert({ name: "Silke", age: 16, phone_id: 1 });
            yield Phone.insert({ number: "123", user_id: 2 });
            yield Phone.insert({ number: "456", user_id: 2 });
            yield builder_1.default.table("phones_users").insert({ phone_id: 1, user_id: 1 }, { phone_id: 2, user_id: 1 });
            const user = (yield User.first());
            const phones = yield user.allPhones();
            chai_1.expect(phones.length).to.equal(2);
            const users = yield phones[0].allUsers();
            chai_1.expect(users.length).to.equal(1);
            const usersAbove20 = yield phones[0].allUsers.where("age", ">=", 20).get();
            chai_1.expect(usersAbove20.length).to.equal(0);
        });
    }
    actsAsQueryBuilder() {
        return __awaiter(this, void 0, void 0, function* () {
            yield User.insert({ name: "Thijs", age: 17, phone_id: 1 });
            yield Phone.insert({ number: "123", user_id: 1 });
            yield Phone.insert({ number: "456", user_id: 1 });
            const user = (yield User.first());
            const allWith3 = yield user.phones.where("number", "LIKE", "%3%").get();
            chai_1.expect(allWith3).to.have.length(1);
            chai_1.expect(allWith3[0].number).to.equal("123");
            chai_1.expect(yield user.phone.count()).to.equal(2);
            chai_1.expect(yield allWith3[0].user.pluck("name")).to.deep.equal(["Thijs"]);
            chai_1.expect(user.phone.foo).to.equal(undefined);
            chai_1.expect(user.phones.foo).to.equal(undefined);
            chai_1.expect(user.allPhones.foo).to.equal(undefined);
            chai_1.expect(allWith3[0].user.foo).to.equal(undefined);
        });
    }
    isLazy() {
        return __awaiter(this, void 0, void 0, function* () {
            yield User.insert({ name: "Thijs", age: 17, phone_id: 1 });
            yield Phone.insert({ number: "123", user_id: 1 });
            yield Phone.insert({ number: "456", user_id: 1 });
            const user = (yield User.first());
            chai_1.expect(yield user.phone()).to.equal(yield user.phone());
            chai_1.expect(yield user.phones()).to.equal(yield user.phones());
            chai_1.expect(yield user.allPhones()).to.equal(yield user.allPhones());
            const phone = (yield Phone.first());
            chai_1.expect(yield phone.user()).to.equal(yield phone.user());
            chai_1.expect(yield phone.hasUser()).to.equal(yield phone.hasUser());
        });
    }
    isImmutable() {
        return __awaiter(this, void 0, void 0, function* () {
            yield User.insert({ name: "Thijs", age: 17, phone_id: 1 });
            yield Phone.insert({ number: "123", user_id: 1 });
            const phone2 = new Phone();
            const phones = yield (yield User.first()).phones();
            chai_1.expect(() => {
                phones.push(phone2);
            }).to.throw("object is not extensible");
        });
    }
};
__decorate([
    mocha_typescript_1.test,
    __metadata("design:type", Function),
    __metadata("design:paramtypes", []),
    __metadata("design:returntype", Promise)
], RelationTests.prototype, "hasOne", null);
__decorate([
    mocha_typescript_1.test,
    __metadata("design:type", Function),
    __metadata("design:paramtypes", []),
    __metadata("design:returntype", Promise)
], RelationTests.prototype, "belongsTo", null);
__decorate([
    mocha_typescript_1.test,
    __metadata("design:type", Function),
    __metadata("design:paramtypes", []),
    __metadata("design:returntype", Promise)
], RelationTests.prototype, "hasMany", null);
__decorate([
    mocha_typescript_1.test,
    __metadata("design:type", Function),
    __metadata("design:paramtypes", []),
    __metadata("design:returntype", Promise)
], RelationTests.prototype, "hasAndBelongsToMany", null);
__decorate([
    mocha_typescript_1.test,
    __metadata("design:type", Function),
    __metadata("design:paramtypes", []),
    __metadata("design:returntype", Promise)
], RelationTests.prototype, "actsAsQueryBuilder", null);
__decorate([
    mocha_typescript_1.test,
    __metadata("design:type", Function),
    __metadata("design:paramtypes", []),
    __metadata("design:returntype", Promise)
], RelationTests.prototype, "isLazy", null);
__decorate([
    mocha_typescript_1.test,
    __metadata("design:type", Function),
    __metadata("design:paramtypes", []),
    __metadata("design:returntype", Promise)
], RelationTests.prototype, "isImmutable", null);
RelationTests = __decorate([
    mocha_typescript_1.suite
], RelationTests);
