"use strict";
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __metadata = (this && this.__metadata) || function (k, v) {
    if (typeof Reflect === "object" && typeof Reflect.metadata === "function") return Reflect.metadata(k, v);
};
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const mocha_typescript_1 = require("mocha-typescript");
const chai_1 = require("chai");
const sqlite3_1 = require("sqlite3");
const _1 = require("../");
const base_model_1 = require("../base-model");
const builder_1 = require("../query/builder");
let WrappingTests = class WrappingTests {
    before() {
        return __awaiter(this, void 0, void 0, function* () {
            let db;
            yield new Promise(resolve => db = new sqlite3_1.Database(":memory:", () => resolve()));
            yield new Promise(resolve => db.exec("CREATE TABLE users (id INTEGER NOT NULL PRIMARY KEY AUTOINCREMENT UNIQUE, name TEXT, age INTEGER)", () => resolve()));
            _1.default.sqlite(db);
        });
    }
    after() {
        return __awaiter(this, void 0, void 0, function* () {
            // Ensure that database is clean unset after our tests.
            _1.default.use(null);
        });
    }
    allowsCustomName() {
        return __awaiter(this, void 0, void 0, function* () {
            class _User extends base_model_1.default {
            }
            const User = _1.default.wrap()(_User, "user");
            chai_1.expect(User.where(x => { }).table).to.equal("user");
        });
    }
    proxiesToQueryBuilder() {
        return __awaiter(this, void 0, void 0, function* () {
            class _User extends base_model_1.default {
            }
            const User = _1.default.wrap()(_User);
            yield builder_1.default.table("users").insert({ name: "Thijs", age: 17 });
            const first = yield User.find(1);
            chai_1.expect(first).to.not.equal(undefined);
            chai_1.expect(first.name).to.equal("Thijs");
            const second = yield User.where("age", 17).first();
            chai_1.expect(second).to.not.equal(undefined);
            chai_1.expect(second.age).to.equal(17);
        });
    }
    "inserts, updates and deletes"() {
        return __awaiter(this, void 0, void 0, function* () {
            class _User extends base_model_1.default {
            }
            const User = _1.default.wrap()(_User);
            const user = new User();
            user.name = "Thijs";
            user.age = 17;
            chai_1.expect(user.id).to.equal(undefined);
            yield user.save();
            chai_1.expect(user.id).to.equal(1);
            user.age = 18;
            yield user.save();
            chai_1.expect(user.id).to.equal(1);
            chai_1.expect(yield User.where("age", 18).count()).to.equal(1);
            yield user.delete();
            chai_1.expect(user.id).to.equal(undefined);
            chai_1.expect(() => {
                user.delete();
            }).to.throw("Cannot delete object if it is not in the database.");
        });
    }
};
__decorate([
    mocha_typescript_1.test,
    __metadata("design:type", Function),
    __metadata("design:paramtypes", []),
    __metadata("design:returntype", Promise)
], WrappingTests.prototype, "allowsCustomName", null);
__decorate([
    mocha_typescript_1.test,
    __metadata("design:type", Function),
    __metadata("design:paramtypes", []),
    __metadata("design:returntype", Promise)
], WrappingTests.prototype, "proxiesToQueryBuilder", null);
__decorate([
    mocha_typescript_1.test,
    __metadata("design:type", Function),
    __metadata("design:paramtypes", []),
    __metadata("design:returntype", Promise)
], WrappingTests.prototype, "inserts, updates and deletes", null);
WrappingTests = __decorate([
    mocha_typescript_1.suite
], WrappingTests);
